/*
 * Copyright (c) 2007, Ryo Ichinose
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright 
 *      notice, this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright 
 *      notice, this list of conditions and the following disclaimer in the 
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of the auther nor the 
 *      names of its contributors may be used to endorse or promote products 
 *      derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <avr/io.h>
#include <inttypes.h>

#include "adc.h"
#include "usart.h"
#include "util.h"


#define INDEX_TO_INSERT_X  9
#define INDEX_TO_INSERT_Y  14
#define INDEX_TO_INSERT_Z  19


static uint16_t get_data(char ch);
static void fill(char *buf, int index, int data);


int
main(void)
{
	char buf[32] = "(X,Y,Z)=(    ,    ,    )\r\n";;

	uint16_t x;
	uint16_t y;
	uint16_t z;

	pullup();
	/* PA1-3 ץ륢å̵ */
	PORTA &= ~(_BV(PINA1) | _BV(PINA2) | _BV(PINA3));

	/* (8MHz/64) = 125KHz */
	adc_init(ADC_CK_64, ADC_NO_INT);
	usart_init(USART_NO_INT);

	while (1){
		x = get_data(PINA3);
		y = get_data(PINA2);
		z = get_data(PINA1);

		fill(buf, INDEX_TO_INSERT_X, x);
		fill(buf, INDEX_TO_INSERT_Y, y);
		fill(buf, INDEX_TO_INSERT_Z, z);

		usart_tx_string(buf);
		delay_500ms();
	}

	return 0; /* ٹ򤱤뤿 */
}


static uint16_t
get_data(char ch)
{
	adc_set_ch(ch);
	adc_start();
	adc_wait();
	return adc_get();
}


static void
fill(char *buf, int index, int data)
{
	char abuf[5];
	char *pos = abuf;
	char ch;

	itoa(abuf, data, 10, 4, '0');

	while ((ch = *(pos ++))) {
		buf[index ++] = ch;
	}
}

/* End of main.c */
